.\" file MASS/lda.d
.\" copyright (C) 1994-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN lda
.TL
Linear Discriminant Analysis
.DN
Linear discriminant analysis.
.CS
lda(formula, data, prior = proportions, tol = 1.0e-4, 
                   subset, na.action = na.fail,
		   method, CV = F, nu)
lda(x,   grouping, prior = proportions, tol = 1.0e-4, 
                   subset, na.action = na.fail,
                   method, CV = F, nu)
.RA
.AG formula
A formula of the form `groups ~ x1 + x2 + ...'  That is, the
response is the grouping factor and the right hand side specifies
the (non-factor) discriminators.
.OA
.AG data
Data frame from which variables specified in  `formula' are
preferentially to be taken.
.AG x
(required if no formula is given as the principal argument.)
a matrix or data frame or Matrix containing the explanatory variables.
.AG grouping
(required if no formula principal argument is given.)
a factor specifying the class for each observation.
.AG prior
the prior probabilities of class membership.  If unspecified, the
class proportions for the training set are used.  If present, the
probabilities should be specified in the order of the factor
levels. 
.AG tol
A tolerance to decide if a matrix is singular; it will reject variables
and linear combinations of unit-variance variables whose variance is
less than `tol^2'.
.AG subset
An index vector specifying the cases to be used in the training
sample.  (NOTE: If given, this argument must be named.)
.AG na.action
A function to specify the action to be taken if `NA's are found.
The default action is for the procedure to fail.  An alternative is
na.omit, which leads to rejection of cases with missing values on
any required variable.  (NOTE: If given, this argument must be named.)
.AG method
`"moment"' for standard estimators of the mean and variance,
`"mle"' for MLEs, `"mve"' to use `cov.mve', or `"t"' for robust 
estimates based on a t distribution.
.AG CV
If true, returns results (classes and posterior probabilities) for
leave-out-out cross-validation. Note that if the prior is estimated,
the proportions in the whole dataset are used.
.AG nu
degrees of freedom for `"method = t"'.
.RT
an object of class `lda' containing the following components:
.RC prior
the prior probabilities used.
.RC means
the group means.
.RC scaling
a matrix which transforms observations to discriminant functions,
normalized so that within groups covariance matrix is spherical.
.RC svd
the singular values, which give the ratio of the between- and
within-group standard deviations on the linear discriminant
variables.  Their squares are the canonical F-statistics.
.RC N
The number of observations used.
.RC call
The (matched) function call.
.PP
unless `CV=T', when the return value is a list with components:
.RC class
The MAP classification (a factor)
.RC posterior
posterior probabilities for the classes
.DT
The function
tries hard to detect if the within-class covariance matrix is
singular. If any variable has within-group variance less than
`tol^2' it will stop and report the variable as constant.  This
could result from poor scaling of the problem, but is more
likely to result from constant variables.
.SH NOTE
This function may be called giving either a formula and
optional data frame, or a matrix and grouping factor as the first
two arguments.  All other arguments are optional, but `subset=' and
`na.action=', if required, must be fully named.

If a formula is given as the principal argument the object may be
modified using `update()' in the usual way.
.SA
`predict.lda', `qda', `predict.qda', `discr'
.EX
> Iris <- data.frame(rbind(iris[,,1], iris[,,2], iris[,,3]), 
+   Sp = rep(c("s","c","v"), rep(50,3)))
> train <- sample(1:150, 75)
> table(Iris$Sp[train])
  c  s  v 
 22 23 30
> z <- lda(Sp ~ ., Iris, prior = c(1,1,1)/3, subset = train)
> predict(z, Iris[-train, ])$class
 [1] s s s s s s s s s s s s s s s s s s s s s s s s s s s c c c
[31] c c c c c c c v c c c c v c c c c c c c c c c c c v v v v v
[61] v v v v v v v v v v v v v v v
> z1 <- update(z, . ~ . - Petal.W.)
.KW multivariate
.WR
