.\" file MASS/rlm.d
.\" copyright (C) 1994-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN rlm
.TL
Fit Robust Linear Regression Model
.DN
Fits a robust linear regression model, using an M-estimator with
Huber's psi function.
.CS
rlm(formula, data, weights, subset, na.action, 
     model=F, k=1.345, sw=1000, ...)
.RA
.AG formula
a formula object, with the response on the left of a `~' operator,
and the terms, separated by `+' operators, on the right.
.OA
.AG data
an optional `data.frame' in which to interpret the variables named in the
formula, or in the `subset' and the `weights' argument.
.AG weights
optional weights; if supplied, the algorithm fits to minimize the sum
of the weights multiplied into the squared residuals.
The weights must be strictly positive.
.AG subset
optional expression saying that only a subset of the rows of the data
should be used in the fit.
.AG na.action
a missing-data filter function, applied to the `model.frame', after
any subset argument has been used.
.AG model
flag to control what is returned.  If this is `TRUE', then the model frame
is returned.  `X' and `y' are always returned.
.AG k
The control value for Winsorizing. The default gives 95% efficiency at the
normal.
.AG sw
switch to Huber proposal 2 scale at iteration `sw' and beyond.
.AG ...
additional arguments for the fitting routines.
The most likely one is `maxit', which sets the iteration limit, by default
20.
.RT
an object of class `rlm' representing the fit, inheriting from `lm'.
This has all the components of an `lm' object, plus `k,' the scale `s' and
`conv' which is a vector monitoring the convergence.
.DT
The fit uses Huber's M-estimator, and initially uses the median
absolute deviation scale estimate based on the residuals. This can be
changed to Huber's proposal 2 after `sw' iterations.
.PP
Generic functions such as `print' and `summary' have methods to
show the results of the fit.
.SA
`summary.rlm'
.EX
attach(phones)
res <- rlm(calls ~ year)
print(res)

rlm(stack.loss ~ stack.x)
.KW robust
.WR
