# file spatial/pp.q copyright (C) 1994-9 W. N. Venables and B. D. Ripley
#
ppinit <- function(file)
{
  tfile <- file
  file.exists <- function(name) {
    if(platform() == "WIN386") access(name, 0) == 0
    else
      unix(paste("test -r", name), output.to.S = F) == 0
  }
  if(exists(".sp.lib.name", f=0)) {
    t1file <- paste(get(".sp.lib.name", f=0), file, sep="/")
    if(file.exists(t1file)) tfile <- t1file
  }
  h <- scan(tfile, list(xl = 0, xu = 0, yl = 0, yu = 0, fac = 0),
	    n = 5, skip = 2)
  pp <- scan(tfile, list(x = 0, y = 0), skip = 3)
  pp$x <- pp$x/h$fac
  pp$y <- pp$y/h$fac
  pp$area <- c(xl=h$xl/h$fac, xu=h$xu/h$fac, yl=h$yl/h$fac, yu=h$yu/h$fac)
  if(!is.loaded(symbol.C("VR_ppset"))) 
    stop("Compiled code has not been dynamically loaded")
  ppregion(pp)
  invisible(pp)
}

Kfn <- function(pp, fs, k = 100)
{
  zz <- (c(range(pp$x), range(pp$y)) - ppgetregion())*c(1,-1,1,-1)
  if(any(zz < 0)) stop("some points outside region")
  z <- .C("VR_sp_pp2",
	  as.single(pp$x),
	  as.single(pp$y),
	  as.integer(length(pp$x)),
	  k1 = as.integer(k),
	  h = single(k),
	  dmin = single(1),
	  lm = single(1),
	  as.single(fs))
  list(y = z$h[1:z$k1], x = (seq(1:z$k1) * fs)/k, k = k, 
       dmin = z$dmin, lm = max(z$dmin, z$lm),
       call=match.call())
}

Kenvl <- function(fs, nsim, ...)
{
  dot.expression <- as.expression(substitute(list(...)))[-1]
  h <- Kfn(pp = eval(dot.expression), fs)
  hx <- h$x
  hu <- h$y
  hl <- h$y
  ha <- h$y^2
  for(i in 2:nsim) {
    h <- Kfn(pp = eval(dot.expression), fs)$y
    hu <- pmax(hu, h)
    hl <- pmin(hl, h)
    ha <- ha + h^2
  }
  list(x = hx, lower = hl, upper = hu, aver = sqrt(ha/nsim),
       call=match.call())
}

Kaver <- function(fs, nsim, ...)
{
  dot.expression <- as.expression(substitute(list(...)))[-1]
  h <- Kfn(pp = eval(dot.expression), fs)
  hx <- h$x
  ha <- h$y^2
  for(i in 2:nsim) {
    h <- Kfn(pp = eval(dot.expression), fs)$y
    ha <- ha + h^2
  }
  list(x = hx, y = sqrt(ha/nsim), call=match.call())
}

ppregion <- function(xl = 0, xu = 1, yl = 0, yu = 1)
{
  if(is.list(xl)) {
    if(length(xl$area)) .C("VR_ppset", as.single(xl$area))
    else .C("VR_ppset", as.single(c(xl$xl, xl$xu, xl$yl, xl$yu)))
  } else .C("VR_ppset", as.single(c(xl, xu, yl, yu)))
  invisible()
}

ppgetregion <- function()
{
  xx <- .C("VR_ppget", z=single(4))$z
  names(xx) <- c("xl", "xu", "yl", "yu")
  xx
}

Psim <- function(n)
{
  z <- .C("VR_pdata",
	  as.integer(n),
	  x = single(n),
	  y = single(n))
  invisible(list(x = z$x, y = z$y, call=match.call()))
}

Strauss <- function(n, c = 0, r)
{
  init <-  0
  if(!exists(".ppx", frame=1)) {
    init <-  1
    z <- .C("VR_pdata",
	    as.integer(n),
	    x = single(n),
	    y = single(n))
    assign(".ppx", z$x, frame=1)
    assign(".ppy", z$y, frame=1)
  }
  z <- .C("VR_simpat",
	  as.integer(n),
	  x = as.single(.ppx),
	  y = as.single(.ppy),
	  as.single(c),
	  as.single(r),
	  as.integer(init))
  assign(".ppx", z$x, frame=1)
  assign(".ppy", z$y, frame=1)
  invisible(list(x = z$x, y = z$y, call=match.call()))
}

SSI <- function(n, r)
{
  z <- .C("VR_simmat",
	  as.integer(n),
	  x = single(n),
	  y = single(n),
	  as.single(r))
  invisible(list(x = z$x, y = z$y, call=match.call()))
}

pplikfn <- function(cc, R, n, x, y, ng, target, trace=F)
{
  z <- .C("VR_plike",
	  as.single(x),
	  as.single(y),
	  as.integer(n),
	  as.single(cc),
	  as.single(R),
	  as.integer(ng),
	  as.single(target),
	  res=single(1)
	  )
  if(trace) print(c(cc, z$res))
  z$res
}

pplik <- function(pp, R, ng=50, trace=F)
{
  n <- length(pp$x)
  ar <- pp$area
  target <- n * (Kfn(pp, R,1)$y)^2 * pi /
    ((ar["xu"] - ar["xl"]) * (ar["yu"] - ar["yl"]))
  if(target == 0) return(0)
  tmp <- pplikfn(1, R, n, pp$x, pp$y, ng, target, F)
  if(tmp <= 0) return(1)
  uniroot(pplikfn, c(0,1), f.lower=-target, f.upper=tmp,
	  R=R, n=n, x=pp$x, y=pp$y, ng=ng, target=target,
	  trace=trace)$root
}

